#!/bin/sh

CURPATH=`pwd`
OUTDIR=
QUIETFLAG=0
COLLDUMPFLAG=0
COLLIPINFOFLAG=0
LOCKFILE="/tmp/HostinfoAnalyzeTool-solaris.lock"

RUN_CONFLICT_EXITCODE=200
NO_ENOUGH_SPACE_EXITCODE=201

CheckSystemType()
{
    TMP_OS_V="`uname -s`"
    if [ "${TMP_OS_V}" != "SunOS" ]
    then
        echo "Only support SunOS! The OS is ${TMP_OS_V}"
        exit 1
    fi
}

LockShell()
{
    if [ "$1" = "" ]
    then
        echo "need lock file param"
        exit 1
    fi

    if [ -f "$1" ]
    then
        pid=`cat $1`
        if [ "${pid}" != "" ]
        then
            PROCOUNT=`ps -ef | awk '{print $2}' |grep -c "^${pid}$"`
            if [ $PROCOUNT = 1 ]
            then
                echo "This shell is already running, exit."
                exit $RUN_CONFLICT_EXITCODE
            fi
        fi
    fi

    echo $$ > $1
    chmod 777 $1
}

UnLockShell()
{
    if [ "$1" = "" ]
    then
        echo "need lock file param"
        return 1
    fi

    if [ -f "${1}" ]
    then
        rm -f "${1}"
    fi

    return 0
}

HostExit()
{
    UnLockShell "${LOCKFILE}"
    exit "$1"
}

PrintYesOrNo()
{
    userInput=""

    echo "$1"

    while [ 1 -eq 1 ]
    do
        echo -n "<Y|N>:"

        read userInput
        if [ "${userInput}" != "yes" ] && [ "${userInput}" != "y" ] && [ "${userInput}" != "no" ] && [ "${userInput}" != "n" ] \
           && [ "${userInput}" != "YES" ] && [ "${userInput}" != "Y" ] && [ "${userInput}" != "NO" ] && [ "${userInput}" != "N" ]
        then
            echo "you select is incorrect[${userInput}]"
            continue
        fi

        if [ "${userInput}" = "yes" ] || [ "${userInput}" = "y" ] || [ "${userInput}" = "YES" ] || [ "${userInput}" = "Y" ]
        then
            return 0
        else
            return 1
        fi
    done

    return 0
}

NotesInformation()
{
    if [ $QUIETFLAG -eq 0 ]
	then
    PrintYesOrNo "WARNING: Do you want to collect dump file? If yes,the dump file may be involved in the collection of private data or suspected by customer,please confirm with the customer before the collection."
    if [ $? -ne 0 ]
        then
            COLLDUMPFLAG=0
        else
    	COLLDUMPFLAG=1
    fi
	fi
	
	if [ $COLLDUMPFLAG -eq 1 ]
	then
    echo  "NOTE: the tool will collect hostInfo as following:
    1) system information
    2) HBA cards information
    3) iscsi information
    4) disk information
    5) UltraPath software information
    6) some important files
    7) dump files "
	else
    echo  "NOTE: the tool will collect hostInfo as following:
    1) system information
    2) HBA cards information
    3) iscsi information
    4) disk information
    5) UltraPath software information
    6) some important files"
	fi
	
    if [ $QUIETFLAG -eq 0 ]
    then
    PrintYesOrNo "Do you want to collect these infomation?"
        if [ $? -ne 0 ]
        then
            rm -f $LOCKFILE
            exit 1
        fi
    fi
}

usage()
{
	printf "\nUsage : $0 [-s] [-f|-d] [-d] "
	printf "\n\t-s\t : silent mode"
	printf "\n\t-s -f\t : collect full information"
	printf "\n\t-s -f -d\t : collect full information and dump information"
	printf "\n\n"
}


GetOutDir()
{
    COUNT=0
    echo "Please input the directory for the collected informations, the default is the current dir. "

    while [ $COUNT -lt 3 ]
    do
        echo -n "Please input the directory : "
        read inputdirectory
        if [ "$inputdirectory" = "" ]
        then
            OUTDIR=$CURPATH/
            return 0
        fi

        if [ ! -d $inputdirectory ]
        then
            if [ $COUNT -lt 2 ]
            then
                echo "Current directory is not exist,please input a valid directory."
             fi

            if [ $COUNT -eq 2 ]
            then
                echo "You have input the invalid directory three times,the collected informations will be saved in the current dir."
                PrintYesOrNo "Are you continue?"
                if [ $? -ne 0 ]
                then
                    rm -f $LOCKFILE
                    exit 1
                fi

                OUTDIR=$CURPATH/
                return 0
            fi

            COUNT=`expr $COUNT + 1`
            continue
        fi

        break
    done

    OUTDIR="$inputdirectory"
}

CheckOutDirFreeSpace()
{
    spaceStr=""
    freeSpace=""
	
    if [ -z "$1" ] || [ -z "$2" ]
    then
        echo "param error[$1,$2]"
        rm -f $LOCKFILE
        exit 1
    fi
    
    spaceStr=`df -k "$1" 2>/dev/null | sed -n '$p'`
    if [ "${spaceStr}" = "" ]
    then
        echo "get disk[$1] free space failed. "
        rm -f $LOCKFILE
        exit 1
    fi

    freeSpace=`df -k "$1" 2>/dev/null | sed -n '$p' | awk '{print $(NF-2)}'`
    if [ -z "${freeSpace}" ] || [ ${freeSpace} -lt $2 ]
    then
        echo  "more disk free space need[$1,$2k]"
        rm -f $LOCKFILE
        exit $NO_ENOUGH_SPACE_EXITCODE
    fi

    return 0
}

CheckOutDirFreeSpaceAndCopy()
{
    spaceStr=""
    freeSpace=""
	
	ID="`id | awk '{print $1}'`"
	if [ "$ID" != "uid=0(root)" ]; then
    return 1
	fi

    if [ -z "$1" ] || [ -z "$2" ]
    then
        echo "param error[$1,$2]"
        rm -f $LOCKFILE
        exit 1
    fi
    
    spaceStr=`df -k "$1" 2>/dev/null | sed -n '$p'`
    if [ "${spaceStr}" = "" ]
    then
        echo "get disk[$1] free space failed. "
        rm -f $LOCKFILE
        exit 1
    fi

    freeSpace=`df -k "$1" 2>/dev/null | sed -n '$p' | awk '{print $(NF-2)}'`
    if [ -z "${freeSpace}" ] || [ ${freeSpace} -lt $2 ]
    then
        echo  "more disk free space need[$1,$2k]"
        rm -f $LOCKFILE
        exit $NO_ENOUGH_SPACE_EXITCODE
    fi
	
    cp -r $ULTRAPATH_DIR_LOG  $BAKFILEDIR 2>>$ERRORREPORTFILE
	
    return 0
}

CheckSystemType
LockShell "${LOCKFILE}"

while [ $# -gt 0 ]
do
    SHFT=2
    case $1 in
    -s)        QUIETFLAG=1;SHFT=1;;
    -f)        COLLFULLFLAG=1;QUIETFLAG=2;SHFT=1;;
    -d)        COLLDUMPFLAG=1;QUIETFLAG=2;SHFT=1 ;;
    *)        echo "\033[1;31mUnknown command line option: ${1}\033[0m";usage;HostExit 0;;
    esac
    shift ${SHFT}
done


NotesInformation

if [ $QUIETFLAG -eq 0 ]
then
    GetOutDir
else
    OUTDIR=$CURPATH
fi

DUMPDIR=`cat /etc/dumpadm.conf | grep "DUMPADM_SAVDIR" | awk -F"=" '{print $2}'`
DUMPSIZE=`du -k $DUMPDIR |awk '{print $1}'`


if [ "$DUMPSIZE" != "" ]
then
    ADDDUMPZIE=`expr $DUMPSIZE + 30000`
else
    ADDDUMPZIE=3000
fi	
CheckOutDirFreeSpace $OUTDIR $ADDDUMPZIE

HOSTROOTDIR="$OUTDIR"/hostinfo
SYSINFOFILE="$HOSTROOTDIR"/sysinfo.txt
HBAINFOFILE="$HOSTROOTDIR"/hbainfo.txt
ISCSIINFOFILE="$HOSTROOTDIR"/iscsiinfo.txt
DISKINFOFILE="$HOSTROOTDIR"/diskinfo.txt
NXUPINFOFILE="$HOSTROOTDIR"/nxupinfo.txt
ERRORREPORTFILE="$HOSTROOTDIR"/error_report

FILESYSTEMINFO="$HOSTROOTDIR"/filesystem.txt
SYSRUNINFO="$HOSTROOTDIR"/sysruninfo.txt
VXVMINFO="$HOSTROOTDIR"/vxvminfo.txt
SVMINFO="$HOSTROOTDIR"/svminfo.txt
SUNCLUSTERINFO="$HOSTROOTDIR"/sunclusterinfo.txt

if [ -d "$HOSTROOTDIR" ]
then
    rm -rf $HOSTROOTDIR/*
	mkdir -p $HOSTROOTDIR
else
    mkdir -p $HOSTROOTDIR
fi

echo "Begin to collect system information."
echo "system information" > $SYSINFOFILE
echo "\n-->uname -a" >> $SYSINFOFILE
uname -a >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->hostname" >> $SYSINFOFILE
hostname >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->id" >> $SYSINFOFILE
id >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->who" >> $SYSINFOFILE
who >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->showrev" >> $SYSINFOFILE
showrev >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->showrev -p" >> $SYSINFOFILE
showrev -p  >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->cat /etc/release" >> $SYSINFOFILE
cat /etc/release >> $SYSINFOFILE  2>>$ERRORREPORTFILE
echo "\n-->isainfo -v" >> $SYSINFOFILE
isainfo -v >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->prtconf" >> $SYSINFOFILE
prtconf >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->prtconf -vp" >> $SYSINFOFILE
prtconf -vp >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->history" >> $SYSINFOFILE
if [ -x history ] ;then 
    history >> $SYSINFOFILE 2>>$ERRORREPORTFILE
fi
echo "\n-->cat /.sh_history" >> $SYSINFOFILE
cat /.sh_history >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->cat /.bash_history" >> $SYSINFOFILE
cat /.bash_history >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->hostid" >> $SYSINFOFILE
hostid >> $SYSINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->ulimit -a" >> $SYSINFOFILE
ulimit -a >> $SYSINFOFILE 2>>$ERRORREPORTFILE

echo "Begin to collect filesystem information"
echo "filesystem information" > $FILESYSTEMINFO
echo "\n-->cat /etc/vfstab" >> $FILESYSTEMINFO
cat /etc/vfstab >> $FILESYSTEMINFO 2>>$ERRORREPORTFILE
echo "\n-->df -h" >> $FILESYSTEMINFO
df -h >> $FILESYSTEMINFO  2>>$ERRORREPORTFILE
echo "\n-->df -k" >> $FILESYSTEMINFO
df -k >> $FILESYSTEMINFO 2>>$ERRORREPORTFILE
echo "\n-->mount" >> $FILESYSTEMINFO
mount >> $FILESYSTEMINFO 2>>$ERRORREPORTFILE

echo "Begin to collect HBA port information."
echo "HBA port information" > $HBAINFOFILE
echo "\n-->fcinfo hba-port" >> $HBAINFOFILE
fcinfo hba-port >> $HBAINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->luxadm -e port" >> $HBAINFOFILE
luxadm -e port >> $HBAINFOFILE 2>>$ERRORREPORTFILE

echo "Begin to collect iscsi information."
echo "iscsi information" > $ISCSIINFOFILE
echo "\n-->iscsiadm list initiator-node" >> $ISCSIINFOFILE
iscsiadm list initiator-node >> $ISCSIINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->pkgparam SUNWiscsir VERSION" >> $ISCSIINFOFILE
pkgparam SUNWiscsir VERSION >> $ISCSIINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->pkgparam SUNWiscsiu VERSION" >> $ISCSIINFOFILE
pkgparam SUNWiscsiu VERSION >> $ISCSIINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->iscsiadm list target" >> $ISCSIINFOFILE
iscsiadm list target >> $ISCSIINFOFILE 2>>$ERRORREPORTFILE

echo "Begin to collect disk information."
echo "disk information" > $DISKINFOFILE
echo "disk" > $HOSTROOTDIR/disk_param
echo "\n-->format -f" >> $DISKINFOFILE
format -f $HOSTROOTDIR/disk_param >> $DISKINFOFILE 2>>$ERRORREPORTFILE
rm -f $HOSTROOTDIR/disk_param
echo "\n-->ls -Llrt /dev/rdsk/" >> $DISKINFOFILE
ls -Llrt /dev/rdsk/ >> $DISKINFOFILE 2>>$ERRORREPORTFILE
echo "\n-->ls -Llrt /dev/dsk/" >> $DISKINFOFILE
ls -Llrt /dev/dsk/ >> $DISKINFOFILE 2>>$ERRORREPORTFILE

echo "Begin to collect system running information"
echo "system running information" > $SYSRUNINFO
echo "\n-->echo ::threadlist -v|mdb -k " >> $SYSRUNINFO
echo "::threadlist -v" | mdb -k >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->echo ::cpuinto|mdb -k " >> $SYSRUNINFO
echo "::cpuinfo" | mdb -k >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->echo ::memstat |mdb -k " >> $SYSRUNINFO
echo "::memstat" | mdb -k >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->echo ::ptree|mdb -k " >> $SYSRUNINFO
echo "::ptree" | mdb -k >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->swap -l " >> $SYSRUNINFO
swap -l >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->sar 2 5 " >> $SYSRUNINFO
sar 2 5  >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->vmstat 2 5 " >> $SYSRUNINFO
vmstat 2 5  >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->vmstat -s " >> $SYSRUNINFO
vmstat -s  >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->psrinfo -vp " >> $SYSRUNINFO
psrinfo -vp >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->env " >> $SYSRUNINFO
env >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->last reboot" >> $SYSRUNINFO
last reboot >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->date -u" >> $SYSRUNINFO
date -u >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->uptime" >> $SYSRUNINFO
uptime >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->pkginfo" >> $SYSRUNINFO
pkginfo >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->ls -l /etc/init.d" >> $SYSRUNINFO
ls -l /etc/init.d >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->ls -l /etc/rc3.d" >> $SYSRUNINFO
ls -l /etc/rc3.d >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->ls -l /boot" >> $SYSRUNINFO
ls -l /boot >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->ls -l /boot/solaris" >> $SYSRUNINFO
ls -l /boot/solaris >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->ls -l /opt" >> $SYSRUNINFO
ls -l /opt >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->crontab -l" >> $SYSRUNINFO
crontab -l >> $SYSRUNINFO 2>>$ERRORREPORTFILE
echo "\n-->modinfo" >> $SYSRUNINFO
modinfo >> $SYSRUNINFO 2>>$ERRORREPORTFILE


echo "Begin to collect UltraPath information."
if [ `pkginfo UltraPath 2>/dev/null|wc -l` -eq 0 ]
then
    echo "not install UltraPath." > $NXUPINFOFILE
else

    VERSION=`upadm show version|cut -d':' -f2|cut -d'.' -f1|head -1`
    SUB_VERSION=`upadm show version|cut -d':' -f2|cut -d'.' -f2|head -1`
    if [ ${VERSION} -ge 6 ] && [ ${SUB_VERSION} -eq 3 ]
    then 
        echo "UltraPath information" > $NXUPINFOFILE
        echo "\n-->pkgparam UltraPath VERSION" >> $NXUPINFOFILE
        pkgparam UltraPath VERSION >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath version" >> $NXUPINFOFILE
        upadm show version >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath upconfig" >> $NXUPINFOFILE
        upadm show upconfig >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath alarmenable" >> $NXUPINFOFILE
        upadm show alarmenable >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath path_reliability_enable" >> $NXUPINFOFILE
        upadm show path_reliability_enable >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath upconfig" >> $NXUPINFOFILE
        upadm show upconfig >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath arrays" >>  $NXUPINFOFILE
        upadm show array >>  $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath vluns" >>  $NXUPINFOFILE
        upadm show vlun type=all >>  $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n->UltraPath vlun array information" >> $NXUPINFOFILE
        upadm show array | grep [0-9][0-9].* | awk '{print $1}' > .arrayid
        INDEX=1
        ID=`sed -n ${INDEX}p .arrayid`
        while [ "$ID" != "" ];
        do
             if [ $ID -le 16383 ]; then
                 upadm show vlun array_id=$ID type=all >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
                 INDEX=`expr $INDEX + 1`
                 ID=`sed -n ${INDEX}p .arrayid`
             fi
        done
        rm .arrayid
        echo "\n->UltraPath vlun detail information" >> $NXUPINFOFILE
        upadm show vlun type=all| grep [0-9][0-9].* | awk '{print $1}' > .vlunid
        INDEX=1
        ID=`sed -n ${INDEX}p .vlunid`    
        ID_LAST_TIME=16383        
        while [ "$ID" != "" ];
        do
             if [ $ID -le 16383 ]; then
                if [ $ID -ne $ID_LAST_TIME ];then
                    upadm show vlun id=$ID type=all >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
                fi
                INDEX=`expr $INDEX + 1`
                ID_LAST_TIME=$ID
                ID=`sed -n ${INDEX}p .vlunid`
             fi
        done
        rm .vlunid    
        echo "\n-->UltraPath paths" >>  $NXUPINFOFILE
        upadm show path >>  $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath supportarraylist" >> $NXUPINFOFILE
        upadm show supportarraylist >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
    elif [ ${VERSION} -ge 6 ]
    then
        echo "UltraPath information" > $NXUPINFOFILE
        echo "\n-->pkgparam UltraPath VERSION" >> $NXUPINFOFILE
        pkgparam UltraPath VERSION >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath version" >> $NXUPINFOFILE
        upadm show version >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath upconfig" >> $NXUPINFOFILE
        upadm show upconfig >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath arrays" >>  $NXUPINFOFILE
        upadm show array >>  $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath vluns" >>  $NXUPINFOFILE
        upadm show vlun >>  $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n-->UltraPath paths" >>  $NXUPINFOFILE
        upadm show path >>  $NXUPINFOFILE 2>>$ERRORREPORTFILE
        echo "\n->UltraPath vlun detail information" >> $NXUPINFOFILE
        upadm show vlun | grep [0-9][0-9].* | awk '{print $1}' > .vlunid
        INDEX=1
        ID=`sed -n ${INDEX}p .vlunid`
        while [ "$ID" != "" ];
        do
             if [ $ID -le 16383 ]; then
                 upadm show vlun id=$ID >> $NXUPINFOFILE 2>>$ERRORREPORTFILE
                 INDEX=`expr $INDEX + 1`
                 ID=`sed -n ${INDEX}p .vlunid`
             fi
        done
        rm .vlunid
        echo "\n-->UltraPath supportarraylist" >> $NXUPINFOFILE
        upadm show supportarraylist >> $NXUPINFOFILE 2 >>$ERRORREPORTFILE
    fi
 
fi

if [ `pkginfo VRTSvxvm 2>/dev/null | wc -l` -ne 0 ]
then
	echo "Begin to collect Vxvm information."
	echo "vxvm information" > $VXVMINFO
	echo "\n-->vxdisk list" >> $VXVMINFO
	vxdisk list >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxdisk -e list" >> $VXVMINFO
	vxdisk -e list >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxdisk -s list" >> $VXVMINFO
	vxdisk -s list >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->disk -o alldgs list" >> $VXVMINFO
	vxdisk -o alldgs list >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxdg list" >> $VXVMINFO
	vxdg list >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxprint -ht" >> $VXVMINFO
	vxprint -ht >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxprint -m rootdg" >> $VXVMINFO
	vxprint -m rootdg >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxdmpadm listctlr all" >> $VXVMINFO
	vxdmpadm listctlr all >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxdmpadm listenclosure all" >> $VXVMINFO
	vxdmpadm listenclosure all >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxdmpdbprint" >> $VXVMINFO
	vxdmpdbprint >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxddladm listjbod" >> $VXVMINFO
	vxddladm listjbod >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->xddladm listsupport" >> $VXVMINFO
	vxddladm listsupport >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxdctl mode" >> $VXVMINFO
	vxdctl mode >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxdctl -c mode" >> $VXVMINFO
	vxdctl -c mode >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxtask list" >> $VXVMINFO
	vxtask list >> $VXVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->vxtranslog -l" >> $VXVMINFO
	vxtranslog -l >> $VXVMINFO 2>>$ERRORREPORTFILE
fi



if [ -x /usr/sbin/metadb ]
then
	echo "Begin to collect SVM information."
	echo "SVM information" > $SVMINFO
	echo "\n-->metstat -t" >> $SVMINFO
	metadb >> $SVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->metstat -t" >> $SVMINFO
	metastat >> $SVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->metstat -t" >> $SVMINFO
	metastat -p >> $SVMINFO 2>>$ERRORREPORTFILE
	echo "\n-->metstat -t" >> $SVMINFO
	metastat -t >> $SVMINFO 2>>$ERRORREPORTFILE
fi

if [ -x /usr/cluster/bin/scstat ]
then
	echo "Begin to collect SUN cluster information."
	echo "SUN cluster information" > $SUNCLUSTERINFO
	echo "\n-->scconf -pvv" >> $SUNCLUSTERINFO
	scconf -pvv >> $SUNCLUSTERINFO 2>>$ERRORREPORTFILE
	echo "\n-->scdpm -p all" >> $SUNCLUSTERINFO
	scdpm -p all >> $SUNCLUSTERINFO 2>>$ERRORREPORTFILE
	echo "\n-->cstat -pvv" >> $SUNCLUSTERINFO
	scstat -pvv >> $SUNCLUSTERINFO 2>>$ERRORREPORTFILE
	echo "\n-->scdidadm -L" >> $SUNCLUSTERINFO
	scdidadm -L >> $SUNCLUSTERINFO 2>>$ERRORREPORTFILE
	echo "\n-->smcwebserver -V" >> $SUNCLUSTERINFO
	smcwebserver -V >> $SUNCLUSTERINFO 2>>$ERRORREPORTFILE
fi




BAKFILEDIR="$HOSTROOTDIR"/bakfile
MESSAGE_FILE="/var/adm/messages*"
ULTRAPATH_DIR_LOG="/opt/UltraPath/log"
XMP_CONF_FILE="/etc/up.conf"
XMP_XML_CONF_FILE="/etc/user_cfg.xml"
PATH_TO_INST_FILE="/etc/path_to_inst"
ETC_SYSTEM_FILE="/etc/system"
SVC_LOG="/var/svc/log"
SVC_LOG_SIZE=`du -k $SVC_LOG |awk '{print $1}'`

if [ $QUIETFLAG -ne 1 ]
then
    echo "->vmstat" >> $SYSINFOFILE
    vmstat >> $SYSINFOFILE 2>>$ERRORREPORTFILE
    echo "->ps -aef" >> $SYSINFOFILE
    ps -aef >> $SYSINFOFILE 2>>$ERRORREPORTFILE
    echo "->prstat" >> $SYSINFOFILE
    prstat 1 1 >> $SYSINFOFILE 2>>$ERRORREPORTFILE

    mkdir -p $BAKFILEDIR
    cp -rf $MESSAGE_FILE  $BAKFILEDIR 2>>$ERRORREPORTFILE
	cp -r $XMP_CONF_FILE  $BAKFILEDIR 2>>$ERRORREPORTFILE
    cp -r $XMP_XML_CONF_FILE $BAKFILEDIR 2>>$ERRORREPORTFILE
    cp -r $PATH_TO_INST_FILE  $BAKFILEDIR 2>>$ERRORREPORTFILE
    cp -r $ETC_SYSTEM_FILE  $BAKFILEDIR 2>>$ERRORREPORTFILE
	
	if [ "$SVC_LOG_SIZE" != "" ]
	then
		CheckOutDirFreeSpace $BAKFILEDIR  $SVC_LOG_SIZE
	fi	
	SVCBAKDIR="$BAKFILEDIR/svc_log"
	if [ ! -d $SVCBAKDIR ]
	then
		mkdir -p $SVCBAKDIR
	fi
	
	cp -rf $SVC_LOG  $SVCBAKDIR 2>>$ERRORREPORTFILE

	if [ `pkginfo VRTSvxvm 2>/dev/null | wc -l` -ne 0 ]
	then
		VXVMLOG="/etc/vx/log"
		VXVMLOGSIZE=`du -k $VXVMLOG |awk '{print $1}'`
		if [ "$VXVMLOGSIZE" != "" ]
		then
		    CheckOutDirFreeSpace $BAKFILEDIR  $VXVMLOGSIZE
		fi
		VXVMBAKDIR="$BAKFILEDIR/vxvm"
		if [ ! -d $VXVMBAKDIR ]
		then
		    mkdir -p $VXVMBAKDIR
		fi
		cp -rf $VXVMLOG $VXVMBAKDIR 2>>$ERRORREPORTFILE
	fi
	
	if [ -x /usr/cluster/bin/scstat ]
	then
	    CLUSTER_LOG="/var/cluster/logs/"
        CLUSTER_LOG_SIZE=`du -k $CLUSTER_LOG |awk '{print $1}'`
		if [ "$CLUSTER_LOG_SIZE" != "" ]
		then
		    CheckOutDirFreeSpace $BAKFILEDIR $CLUSTER_LOG_SIZE
		fi
		CLUSTERBAKDIR="$BAKFILEDIR/cluster"
		if [ ! -d $CLUSTERBAKDIR ]
		then
		    mkdir -p $CLUSTERBAKDIR
		fi
        cp -rf $CLUSTER_LOG $CLUSTERBAKDIR 2>>$ERRORREPORTFILE
	fi
	
	if [ $COLLDUMPFLAG -eq 1  ]
	then
        DUMPBAKDIR="$BAKFILEDIR/dump"
		if [ ! -d $DUMPBAKDIR ]
		then
		    mkdir -p $DUMPBAKDIR
		fi
		CheckOutDirFreeSpace $DUMPBAKDIR  $DUMPSIZE
        cp -rf $DUMPDIR $DUMPBAKDIR 2>>$ERRORREPORTFILE
	fi
	
	KERNEL_DRV_BAK="$BAKFILEDIR"/kernel_drv
	if [ ! -d $KERNEL_DRV_BAK ]
	then
		mkdir -p $KERNEL_DRV_BAK
	fi
    cp -rf /kernel/drv/*.conf $KERNEL_DRV_BAK/ 2>>$ERRORREPORTFILE
	
    cp -rf /var/svc/manifest/application/upservice.xml $BAKFILEDIR/upservice.xml 2>>$ERRORREPORTFILE
    
    NXUPLOG_SPACE=`du -k $ULTRAPATH_DIR_LOG |awk '{print $1}'`
    CheckOutDirFreeSpaceAndCopy $BAKFILEDIR $NXUPLOG_SPACE
fi

echo "Begin to commprss files,it may take a few minutes"
cd $OUTDIR/hostinfo
TAR_FILE="Solaris-Info.tar.gz"
tar cf $OUTDIR/$TAR_FILE * 2>/dev/null
if [ $? -ne 0 ]
then
    echo "compress files failed."
    rm -f $LOCKFILE
    exit 1
fi

cd $CURPATH
rm -rf $HOSTROOTDIR
rm -f $LOCKFILE
echo "Collect hostinfo successfully,the HostInfo is in $HOSTROOTDIR/$TAR_FILE."
exit 0
