# coding=utf-8
# 获取linux主机的HBA卡信息
import re
import sys
import os

path = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(path, "..\\..")
sys.path.append(path)
from common.contentParse import check_lst,getStringRet
from common.util import log
from common import util
from common import constants
CLI = None
LANGUAGE = None
CHECK_FAIL = 0
CHECK_PASS = 1
cmd_info_id = ["cmd_info_hba_card_lscfg" ,
               'cmd_info_hab_card_fc_check',
               "cmd_info_hba_card_lsattr",
               "cmd_info_hba_card_status",
               "cmd_info_hba_card_status_D",
               "cmd_info_hba_card_tmout_cfg"]
            
cmd_info_desc = ["lscfg -vpl fcs*",
                 'lsdev -Cc adapter |grep FC',
                 "lsattr -El "    ,
                 "fcstat ", 
                 "fcstat -D ",
                 "lsattr -El "]


fcCmdId = []
fcCmdDesc = []
            
cmd_info_hba_list = "lscfg -vpl fcs*|grep fcs|awk '{print $1}'"
cmd_info_hba_port_list = "lspath |grep fscsi|awk '{print $3}' |awk '!a[$0]++'"
cmdInfoHbaPortList = "lspath |grep -i fscsi|grep -i enabled"
cmdPartHbaStatus = ' | grep Attention'

checkFcCmd = 'lsdev -Cc adapter |grep FC'
drFwCmd = 'lsmcode -A |grep '

def execute(context):
    """
    Function name      : execute
    Function describe  : 外部接入
    Input              : context
    Return             : cmd display
    """
    # 给全局变量赋初始值，按照要求传递上下文进去。
    global CLI
    CLI = context.get("SSH")
    global LANGUAGE
    LANGUAGE = context.get("lang")
    cmd_display = context.get("ret_map")
    
    allFcLst = getAllFcHbas(CLI,checkFcCmd,context)
    for fcHba in allFcLst:
        cmd_info_id.append('cmd_info_hba_card_fc_'+fcHba)
        cmd_info_desc.append(drFwCmd + fcHba)
    
    return check_and_get_result(cmd_execute(cmd_display, CLI, LANGUAGE, context))



def getAllFcHbas(cli,checkCmd,context):
    '''
    fcs0    Available 05-02 10 Gb FCoE PCI Express Dual Port Adapter (771001801410af03)
    fcs1    Available 05-03 10 Gb FCoE PCI Express Dual Port Adapter (771001801410af03)
    fcs2    Available 06-00 4Gb FC PCI Express Adapter (df1000fe)
    fcs3    Available 06-01 4Gb FC PCI Express Adapter (df1000fe)
          需要保证每条记录包含FC PCI字段。
    '''
    linestr = cli.execCmdHasLogTimout(checkCmd,60)
    util.updateItemProgress(context, constants.PROG5)
    lineLst = linestr.splitlines()
    lineLst = check_lst(lineLst)
    lineLst = lineLst[1:]
    fcLst = []
    for line in lineLst:
        if ('FC' in line) and ('FCoE' not in line): 
            lst = re.split('\s+',line.strip())
            fcLst.append(lst[0].strip())
    return fcLst
            


# 取中间一个有效值
def check_and_get_result((fun_flag, result, funErrMsg)):
    """
    :rtype : object
    """
    return result




def handleCmdResult(cmd_display, columns, resultKey, cmdExecResult):
    '''
    @summary: 处理命令结果，整理返回HBA卡超时信息
    @param cmd_display: 命令显示Map
    @param resultKey: 单个命令的显示key
    @return: 
           hbaCardInfoDict HBA卡超时参数对象
    '''
    cmd_display.put(resultKey, cmdExecResult)
    cmdExecResultChanged = cmdExecResult.replace("+", "")
    findResult = re.findall(r'dyntrk\s+yes[.\s].+fc_err_recov\s+fast_fail', cmdExecResultChanged, re.S)
    hbaCardInfoDict = {"Host":{"PortID":"NA"}, "Timeout":"NA"}
    hbaCardInfoDict["Host"]["PortID"] = columns[2]
    hbaCardInfoDict["Timeout"] = "changed" if len(findResult) > 0 else "unchanged"
    return hbaCardInfoDict

def cmd_execute(cmd_display, CLI, LANGUAGE, context):
    fun_flag = CHECK_FAIL
    funErrMsg = ''
    util.updateItemProgress(context, constants.PROG5)
    hba_list = CLI.execCmdHasLogTimout(cmd_info_hba_list, 60).splitlines()
    fcHbaPortCmdRtn = CLI.execCmdWithNoCheckResult(cmdInfoHbaPortList, 60, None)
    util.updateItemProgress(context, constants.PROG10)
    fcHbaPortList = fcHbaPortCmdRtn.splitlines()
    totalCmd = len(cmd_info_id) + len(hba_list) + len(hba_list)
    prgStep = constants.PROG80 / totalCmd
    curstep = constants.PROG10
    for rg in range(len(cmd_info_id)):
        if cmd_info_id[rg] == "cmd_info_hba_card_lsattr":
            for hba in hba_list[1:]:
                cmd_display_temp = CLI.execCmdHasLogTimout(cmd_info_desc[rg] + hba, 60)
                curstep += prgStep
                util.updateItemProgress(context, curstep)
                cmd_display.put("cmd_display" + (cmd_info_id[rg])[8:] + "_" + hba, cmd_display_temp)
                if None == cmd_display_temp or '' == cmd_display_temp or cmd_display_temp.find('TOOLKIT_SEND_CMD_TIME_OUT') > 0 or cmd_display_temp.find('TOOLKIT_EXE_CMD_FAILED') > 0:
                    if "en" == LANGUAGE:
                        funErrMsg += cmd_info_desc[rg] + hba + ":\texecute failed\r\n"
                    else:
                        funErrMsg += cmd_info_desc[rg] + hba + u":\t执行失败\r\n"
                else:
                    if "en" == LANGUAGE:
                        funErrMsg += cmd_info_desc[rg] + hba + ":\texecute success\r\n"
                    else:
                        funErrMsg += cmd_info_desc[rg] + hba + u":\t执行成功\r\n"
                        
        elif cmd_info_id[rg] == "cmd_info_hba_card_status" or cmd_info_id[rg] == "cmd_info_hba_card_status_D":
            for hba in hba_list[1:]:
                cmd_display_temp=CLI.execCmdHasLogTimout(cmd_info_desc[rg] + hba + cmdPartHbaStatus,60)
                statusCmd = "status_" if cmd_info_id[rg] == "cmd_info_hba_card_status" else "status_D_"
                curstep += prgStep
                util.updateItemProgress(context, curstep)
                cmd_display.put("cmd_display" + (cmd_info_id[rg])[8:] + "_" + statusCmd + hba, cmd_display_temp)
                if None == cmd_display_temp or '' == cmd_display_temp or cmd_display_temp.find('TOOLKIT_SEND_CMD_TIME_OUT') > 0 or cmd_display_temp.find('TOOLKIT_EXE_CMD_FAILED') > 0:
                    if "en"==LANGUAGE:
                        funErrMsg        +=cmd_info_desc[rg] + hba + cmdPartHbaStatus+ ":\texecute failed\r\n"
                    else:
                        funErrMsg        +=cmd_info_desc[rg] + hba + cmdPartHbaStatus+  u":\t执行失败\r\n"
                else:
                    if "en"==LANGUAGE:
                        funErrMsg        +=cmd_info_desc[rg] + hba + cmdPartHbaStatus+  ":\texecute success\r\n"
                    else:
                        funErrMsg        +=cmd_info_desc[rg] + hba + cmdPartHbaStatus+  u":\t执行成功\r\n"
                        
        elif cmd_info_id[rg] == "cmd_info_hba_card_tmout_cfg":
            try:
                hbaCardInfoList = []
                if "command not found" in fcHbaPortCmdRtn:
                    hbaCardInfoList.append({"HBATimeout":"NA"})
                    getStringRet(context, str(hbaCardInfoList).replace("u\'", "\'"))
                    continue
                if fcHbaPortCmdRtn.find("hdisk") == -1:
                    hbaCardInfoList.append({"HBATimeout":"not involved"})
                    getStringRet(context, str(hbaCardInfoList).replace("u\'", "\'"))
                    continue
                cmdInfoId = cmd_info_id[rg]
                cmdInfoDesc = cmd_info_desc[rg] 
                #遍历HBA卡处理HBA卡超时参数信息
                funErrMsg = ineratorToHandleHbaTmoutInfo(fcHbaPortList, funErrMsg, context, cmdInfoId, cmdInfoDesc)
                curstep += prgStep
                util.updateItemProgress(context, curstep)
            except Exception, exception:
                log.error(context, "exec fc hbc timeout querry error.exception:%s" % str(exception))
        else:
            cmd_display_temp = CLI.execCmdHasLogTimout(cmd_info_desc[rg], 60)
            curstep += prgStep
            util.updateItemProgress(context, curstep)
            cmd_display.put("cmd_display" + (cmd_info_id[rg])[8:], cmd_display_temp)
            if None == cmd_display_temp or '' == cmd_display_temp or cmd_display_temp.find('TOOLKIT_SEND_CMD_TIME_OUT') > 0 or cmd_display_temp.find('TOOLKIT_EXE_CMD_FAILED') > 0:
                if "en" == LANGUAGE:
                    funErrMsg += cmd_info_desc[rg] + ":\texecute failed\r\n"
                else:
                    funErrMsg += cmd_info_desc[rg] + u":\t执行失败\r\n"
            else:
                if "en" == LANGUAGE:
                    funErrMsg += cmd_info_desc[rg] + ":\texecute success\r\n"
                else:
                    funErrMsg += cmd_info_desc[rg] + u":\t执行成功\r\n"
    cmd_display.put("err_msg", funErrMsg)
    fun_flag = CHECK_PASS
    util.updateItemProgress(context, constants.PROG90)
    return fun_flag, cmd_display, funErrMsg


def handleResultMsg(funErrMsg, execCmd, cmdExecResult):
    '''
    @summary: 处理结果提示信息
    @param funErrMsg: 命令结果提示信息
    @param cmdExecResult: 命令处理结果
    @return: 
           funErrMsg 命令回显结果
    '''
    if None == cmdExecResult or '' == cmdExecResult or cmdExecResult.find('TOOLKIT_SEND_CMD_TIME_OUT') > 0 or cmdExecResult.find('TOOLKIT_EXE_CMD_FAILED') > 0:
        if "en" == LANGUAGE:
            funErrMsg += execCmd + ":\texecute failed\r\n"
        else:
            funErrMsg += execCmd + u":\t执行失败\r\n"
    else:
        if "en" == LANGUAGE:
            funErrMsg += execCmd + ":\texecute success\r\n"
        else:
            funErrMsg += execCmd + u":\t执行成功\r\n"
    return funErrMsg

def ineratorToHandleHbaTmoutInfo(fcHbaPortList, funErrMsg, context, cmdInfoId, cmdInfoDesc):
    '''
    @summary: 遍历HBA卡，查询HBA卡超时参数信息
    @param fcHbaPortList: 端口列表
    @param funErrMsg: 命令处理结果提示信息
    @param context: 命令处理结果提示信息
    @param cmdInfoId: 命令信息ID
    @param cmdInfoDesc: 命令信息提示
    @return: 
           funErrMsg 命令回显结果
    '''
    hbaCardInfoList = []
    cmd_display = context.get("ret_map")
    for hbaPort in fcHbaPortList[1:]:
        if "fscsi" not in hbaPort:
            log.warn(context,"not fc hba card:" + hbaPort)
            continue
        columns = hbaPort.split()
        resultKey = "cmd_display" + (cmdInfoId)[8:] + "_" + columns[2]
        if None == cmd_display.get(resultKey):
            execCmd = cmdInfoDesc + columns[-1]
            #发送命令查询具体FC HBA的超时信息
            cmdExecResult = CLI.execCmdWithNoCheckResult(execCmd, 60, None)
            hbaCardInfoDict = handleCmdResult(cmd_display, columns, resultKey, cmdExecResult)
            hbaCardInfoList.append(hbaCardInfoDict)
            funErrMsg = handleResultMsg(funErrMsg, execCmd, cmdExecResult)
        cmdExecResult = CLI.execCmdWithNoCheckResult(execCmd, 60, None)
    getStringRet(context, str(hbaCardInfoList).replace("u\'", "\'"))
    return funErrMsg
