# coding=utf-8

from com.huawei.ism.tool.infograb.context import EvalResultEnum
from com.huawei.ism.tool.infograb.context import ItemEvalResult
from com.huawei.ism.tool.infograb.entity import Item

import re
import sys
import traceback
import os
path = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(path, "..\\..")
sys.path.append(path)
from common import util
from common import contentParse
from common import constants
def execute(context):
    """
    Function name      : execute
    Function describe  : 外部接入
    Input              : context
    Return             : cmd display{}
    cmd_display.put("cmd_display"+(cmd_info_id[rg])[8:], cmd_display_temp)
    cmd_display.put("err_msg", fun_err_msg  + cmd_display.get("err_msg"))
    cmd_display.put("evl_ret", CHECK_FAIL)
    """
    global CLI
    CLI = context.get("SSH")
    global LANGUAGE
    LANGUAGE = context.get("lang")
    cmd_display = context.get("ret_map")
    util.updateItemProgress(context, constants.PROG5)
    check_and_get_result(cmd_execute(cmd_display, CLI, LANGUAGE))

    global ULTRAPATHCTRL
    ULTRAPATHCTRL = True
    global MULTIPATHCTRL
    MULTIPATHCTRL = True
    global COEXISTENCE
    COEXISTENCE = False
    return doEvaluate(context)

def doEvaluate(context):
    '''
    @summary: Evaluate ultrapath.
    @param context: Python execution context.
    @return: retDict from python execution context.
    '''
    global ULTRAPATHCTRL
    global MULTIPATHCTRL
    global COEXISTENCE
    retDict = context.get("ret_map")
    isQrySucc, ultrapathVersion, cliRet = util.qryUltrapathVersion(context)
    if not isQrySucc:
        errParas = [constants.QRY_ULTRAPATH_CMD, "messageKey:" + constants.ERR_MSG_KEY_QRY_MULTIPATH_FAILED]
        util.log.error(context, 'Query ultrapath version failed.')
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                         constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION, errParas)
        retDict['evalResult'] = itemEvalResult
        return retDict
    else:
        util.log.debug(context, 'Ultrapath version is:' + unicode(ultrapathVersion))
    isQrySucc, isMultipathdRunning, pathdCliRet = util.qryMultipathdStatus(context)
    contentParse.setCmdRet4Report(context, pathdCliRet, "cmd_display_multipathd_status")
    util.updateItemProgress(context, constants.PROG10)
    cliRet += pathdCliRet
    if not isQrySucc:
        errParas = [constants.QRY_MULTIPATHD_STATUS, "messageKey:" + constants.ERR_MSG_KEY_QRY_MULTIPATHD_FAILED]
        util.log.error(context, 'Query multipathd status failed.')
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                         constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION, errParas)
        retDict['evalResult'] = itemEvalResult
        util.updateItemProgress(context, constants.PROG90)
        return retDict

    #None-root user has no privilege to query multipathd running status on Linux of some release version.
    if 'may require superuser privileges' in pathdCliRet:
        util.log.error(context, 'No privilege to query multipathd status.')
        errParas = [constants.QRY_MULTIPATHD_STATUS, "messageKey:" + constants.ERR_MSG_KEY_QRY_MULTIPATHD_NO_PRIVILEGE]
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                         constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION, errParas)
        retDict['evalResult'] = itemEvalResult
        util.updateItemProgress(context, constants.PROG90)
        return retDict

    if ultrapathVersion and isMultipathdRunning:
        COEXISTENCE = True
        util.log.error(context, 'Ultrapath and multipathd service are running at the same time.')
        multipathCoexistenceInfo = util.MultipathCoexistenceInfo()
        multipathCoexistenceInfo.setHostIP(CLI.getHost())
        multipathCoexistenceInfo.setCoexistence(True)
#       执行自研多路径
        itemEvalResultUltrapath = checkRedundantLinkByUltrapath(context, ultrapathVersion)
        multiPathExecuteInfo = setMultipathCoexistenceInfo(cliRet, itemEvalResultUltrapath, ULTRAPATHCTRL)
        multipathCoexistenceInfo.setHuaweiMultipath(multiPathExecuteInfo)
#       执行自带多路径
        itemEvalResultMultipathd = checkRedundantLinkByMultipathd(context)
        multiPathExecuteInfoForSelf = setMultipathCoexistenceInfo(cliRet, itemEvalResultMultipathd, MULTIPATHCTRL)
        multipathCoexistenceInfo.setSelfMultiPath(multiPathExecuteInfoForSelf)
        util.updateItemProgress(context, constants.PROG90)
        contentParse.addMultiPathResultToFile(retDict, multipathCoexistenceInfo.getJsonStr())
        return

    if not ultrapathVersion and not isMultipathdRunning:
        errMsgKey = constants.ERR_MSG_KEY_ULTRAPATH_MULTIPATHD_ALL_NOT_RUNING
        util.log.error(context, 'Neither Ultrapath nor multipathd service is running.')
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet, errMsgKey)
        retDict['evalResult'] = itemEvalResult
        util.updateItemProgress(context, constants.PROG90)
        return retDict

    if ultrapathVersion:#For HUAWEI Ultrapath check.
        itemEvalResult = checkRedundantLinkByUltrapath(context, ultrapathVersion)
        itemEvalResult.setCliRet(cliRet + itemEvalResult.getCliRet())
        util.updateItemProgress(context, constants.PROG90)
        retDict['evalResult'] = itemEvalResult
        return itemEvalResult
    else:#For Linux multhpathd check.
        itemEvalResult = checkRedundantLinkByMultipathd(context)
        itemEvalResult.setCliRet(cliRet + itemEvalResult.getCliRet())
        util.updateItemProgress(context, constants.PROG90)
        retDict['evalResult'] = itemEvalResult
        return itemEvalResult

def setMultipathCoexistenceInfo(cliRet, itemEvalResult, isControl):
    '''
    @summary: set ultrapath information for coexistence.
    @param cliRet: cliRet.
    @param itemEvalResult: multipath result.
    @return: multiPathExecuteInfo.
    '''
    multiPathExecuteInfo = util.MultiPathExecuteInfo()

    itemEvalUltrapathEvalResult = itemEvalResult.getEvalResult()
    itemEvalUltrapathCliRet = cliRet + itemEvalResult.getCliRet()
    itemEvalUltrapathErrParas = itemEvalResult.getErrParas()

#   设置到多路径对象中
    multiPathExecuteInfo.setCliRet(itemEvalUltrapathCliRet)
    multiPathExecuteInfo.setFailCommand('')
    multiPathExecuteInfo.setfailDescribe('')
    multiPathExecuteInfo.setNoRedundantPath('')

    if EvalResultEnum.PASSED == itemEvalUltrapathEvalResult:
        multiPathExecuteInfo.setControl(True)
    else:
        hasfailCommand = False
        for errParas in itemEvalUltrapathErrParas:
            if "messageKey:" in errParas:
                hasfailCommand = True
                break
        multiPathExecuteInfo.setControl(isControl)

        if hasfailCommand:
            multiPathExecuteInfo.setFailCommand(itemEvalUltrapathErrParas[0])
            multiPathExecuteInfo.setfailDescribe(itemEvalUltrapathErrParas[1][11:])
        elif len(itemEvalUltrapathErrParas) > 1:
            multiPathExecuteInfo.setNoRedundantPath(",".join(itemEvalUltrapathErrParas[1:]))

    return multiPathExecuteInfo

def checkRedundantLinkByUltrapath(context, ultrapathVersion):
    '''
    @summary: Check all LUN redundant link with ultrapath.
    @param context: Python execution context.
    @param ultrapathVersion: Ultrapath version
    @return:  item evaluate result of Java bean.
    '''
    tupVer = util.getUltrapathIntVer(context, ultrapathVersion)
    lowerTupVer = util.getUltrapathIntVer(context, constants.LINUX_LOWER_ULTRAPATH_VER)
    middleTupVer = util.getUltrapathIntVer(context, constants.LINUX_MIDDLE_ULTRAPATH_VER)
    if tupVer <= lowerTupVer:
        return checkLowerVerRedundantPath(context)
    elif tupVer <= middleTupVer:
        return checkVlunRedundantLink(context, True)
    else:
        return checkVlunRedundantLink(context, False)

def checkLowerVerRedundantPath(context):
    '''
    @summary: Check all LUN redundant link with ultrapath with version lower than or equal to 5.01.017.
    @param context: Python execution context.
    @return: object of ItemEvalResult 
    '''
    global ULTRAPATHCTRL
    isQrySucc, arrayIDList, cliRet = util.getArrayIdsManagedByUltrapath(context)
    util.addItemProgress(context, getPropProgress(constants.PROG15))
    if not isQrySucc:
        util.log.error(context, 'Query array managed by ultrapath failed.')
        ULTRAPATHCTRL = False
        errParas = [constants.QRY_ARRAY_ID_CMD, "messageKey:" + constants.ERR_MSG_KEY_QRY_ARRAY_ID_FAILED]
        errMsgKey = constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION
        return genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet, errMsgKey, errParas)
    if not arrayIDList:
        util.log.error(context, 'Query no array managed by ultrapath.')
        ULTRAPATHCTRL = False
        errParas = [constants.QRY_ARRAY_ID_CMD]
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                                         cliRet,
                                         constants.ERR_MSG_KEY_QRY_VLUN_ID_EMPTY, errParas)

    cli = context.get("SSH")
    COEXIST_RATE = 2
    progRate = COEXIST_RATE if COEXISTENCE else 1
    prgStep = constants.PROG35 * progRate / len(arrayIDList)
    noRedundantPathLunIds = []
    for arrayId in arrayIDList:
        try:
            arrayLunRet = cli.execCmdWithNoCheckResult('upadm show lun array=' + arrayId,
                                           constants.HOST_CMD_TIMEOUT,
                                           constants.LINUX_CMD_DFT_END_LIST)
            util.addItemProgress(context, prgStep)
            cliRet += arrayLunRet
            lunPathDict = parseAllLunPathOfOneArray(arrayLunRet)

            if not lunPathDict:
                ULTRAPATHCTRL = True
                util.log.error(context, 'No lun found for array:' + unicode(arrayId))
                errMsgKey = constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION
                errParas = ['upadm show lun array=' + arrayId, "messageKey:" + constants.ERR_MSG_KEY_QRY_ARRAY_LUN_PATH_FAILED]
                return genEvalResultForMultipath(EvalResultEnum.FAILED,
                                                 cliRet,
                                                 errMsgKey, errParas)

            noRedundantPathLunIds.extend(chkAllLunRedundantPath(context, lunPathDict))
        except:
            errMsgKey = constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION
            errParas = ['upadm show lun array=' + arrayId, "messageKey:" + constants.ERR_MSG_KEY_QRY_ARRAY_LUN_PATH_FAILED]
            util.log.error(context, 'Query array IDs exception:' + traceback.format_exc())
            return genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                                             errMsgKey, errParas)

    if not noRedundantPathLunIds:
        return genEvalResultForMultipath(EvalResultEnum.PASSED, cliRet, '')
    else:
        errParas = [str(cli.getHost())]
        errParas.append(','.join(noRedundantPathLunIds))
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                         cliRet,
                         constants.ULTRA_NOT_PASS_TIPS,
                         errParas)

def parseAllLunPathOfOneArray(arrayLunRet):
    '''
    @summary: Parse all LUN path of one array according the upadm show lun array = arrayId command result.
    @param arrayLunRet: command result.
    @return: LUN PATH dictionary.
    '''
    lunPathDict = {}
    keyWord = 'Information of Lun'
    if 'Lun Information' in arrayLunRet:
        keyWord = 'Lun Information'
    keyWordLen = len(keyWord)
    lunInfoStartIdx = arrayLunRet.find(keyWord)
    newPathPattern = re.compile(constants.NEW_PATH_FOUND_NEW_PATTERN, re.I)
    pathPattern = re.compile(constants.CTRL_ID_NEW_RE_PATTERN, re.I)
    totLen = len(arrayLunRet)

    while -1 != lunInfoStartIdx and lunInfoStartIdx < totLen - 1:
        nextLunInfoStartIdx = arrayLunRet.find(keyWord, lunInfoStartIdx + keyWordLen)
        lunPath, lunId = '', ''
        for line in arrayLunRet[lunInfoStartIdx:nextLunInfoStartIdx].splitlines():
            line = line.strip().lower()
            if not isValidLunPathInfoLine(line):
                continue

            #find a new LUN
            if 'lun #' in line:
                lunId = line.split('-')[0].split('lun #')[-1].strip()
                lunPathDict[lunId] = []
                continue
            elif 'information of lun' in line:
                lunId = line.split('-')[0].split('information of lun')[-1].strip()
                lunPathDict[lunId] = []
                continue

            #new lun path found
            if newPathPattern.search(line) and pathPattern.search(line):
                lunPath = pathPattern.search(line).group()[1:]
                continue

            if lunPath and line.startswith('numlunobjects'):
                objNum = int(line.split(':')[1].split()[0].strip())
                if objNum > 0:
                    pass
                else:
                    lunPath = ''
                continue

            if lunPath and 'devstate' in line and len(line.split(':')) <= 2:
                devState = line.split(':')[-1].strip()
                if 'optimal' == devState.lower():
                    if lunId in lunPathDict:
                        lunPathDict[lunId].append(lunPath)

        lunInfoStartIdx = nextLunInfoStartIdx
    return lunPathDict

def isValidLunPathInfoLine(line):
    lowerLine = line.lower()
    return ('lun name' in lowerLine or 'controller' in line
         or 'numlunobjects' in line or ('devstate' in line and 'bootowningpath' not in line))

def chkAllLunRedundantPath(context, lunPathDict):
    noRedundantPathLunIds = []
    for lunId in lunPathDict:
        lunPath = lunPathDict.get(lunId)
        if not util.hasRedundantPathForCtrlIdPath(context, lunPath):
            noRedundantPathLunIds.append(lunId)
    return noRedundantPathLunIds

def checkVlunRedundantLink(context, isMiddleVer = True):
    '''
    @summary: Check all LUN redundant link with ultrapath with version higher than 5.01.017
    @param context: Python execution context.
    @return: object of ItemEvalResult 
    '''
    global ULTRAPATHCTRL
    isQrySucc, vlunIdList, cliRet = util.getVlunIdsManagedByUltrapath(context, isMiddleVer)
    util.addItemProgress(context, getPropProgress(constants.PROG15))
    if "can't find any" in cliRet.lower():
        util.log.error(context, 'Query no array managed by ultrapath.')
        ULTRAPATHCTRL = False
        errParas = [constants.QRY_VLUN_ID_CMD if isMiddleVer else constants.High_VER_QRY_VLUN_ID_CMD]
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                                         cliRet,
                                         constants.ERR_MSG_KEY_QRY_VLUN_ID_EMPTY, errParas)

    if not isQrySucc or not vlunIdList:
        util.log.error(context, 'Query vluns managed by ultrapath failed.')
        ULTRAPATHCTRL = False
        errParas = [constants.QRY_VLUN_ID_CMD if isMiddleVer else constants.High_VER_QRY_VLUN_ID_CMD,
                    "messageKey:" + constants.ERR_MSG_KEY_QRY_VLUN_ID_FAILED]
        errMsgKey = constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                                  cliRet, errMsgKey, errParas)

    cli = context.get("SSH")

    noRedundantPathLunIdList = []
    hyperMetroLunNoRedundantIdList = []
    progRate = 1 if COEXISTENCE else 2
    prgStep, stepsUnit = util.calcPerStepDetail(constants.PROG35 * progRate, len(vlunIdList))
    for idx in range(len(vlunIdList)):
        try:
            vlunId = vlunIdList[idx]
            qryVlunPthCmd = 'upadmin show vlun id=' + vlunId + ('' if isMiddleVer else ' type=all')
            vlunPthRet = cli.execCmdWithNoCheckResult(qryVlunPthCmd,
                                                      constants.HOST_CMD_TIMEOUT,
                                                      constants.LINUX_CMD_DFT_END_LIST)
            cliRet += vlunPthRet
            isHyperMetroLUN, chkPass = util.chkSingleLunPathRedundant(context, vlunPthRet)
            util.itemProgressIncr(context, idx, prgStep, stepsUnit)

            if not isHyperMetroLUN:
                if not chkPass:
                    util.log.error(context, 'Normal Vlun path not redundant:' + vlunId)
                    noRedundantPathLunIdList.append(vlunId)
                else:
                    util.log.debug(context, 'Normal or migration Vlun path redundant:' + vlunId)
            elif chkPass:
                util.log.debug(context, 'Hypermetro Vlun path redundant:' + vlunId)
            else:
                hyperMetroLunNoRedundantIdList.append(vlunId)
                util.log.error(context,
                               'Hypermetro Vlun path not redundant '
                               'or from different array or missing sub LUN:' + vlunId)
        except:
            util.log.error(context, 'Query VLUN path exception:' + traceback.format_exc())
            errMsgKey = constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION
            errParas = ['upadmin show vlun id=' + vlunId + ('' if isMiddleVer else ' type=all'),
                        "messageKey:" + constants.ERR_MSG_KEY_QRY_SINGLE_LUN_PATH_FAILED]
            return genEvalResultForMultipath(EvalResultEnum.FAILED,
                                  cliRet, errMsgKey, errParas)

    if not noRedundantPathLunIdList and not hyperMetroLunNoRedundantIdList:
        return genEvalResultForMultipath(EvalResultEnum.PASSED, cliRet, '')
    else:
        errParas = [str(cli.getHost())]
        if noRedundantPathLunIdList and not hyperMetroLunNoRedundantIdList:
            errParas.append(','.join(noRedundantPathLunIdList))
            return genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                                         constants.ULTRA_NOT_PASS_TIPS,
                                         errParas)
        elif not noRedundantPathLunIdList and hyperMetroLunNoRedundantIdList:
            errParas.append(','.join(hyperMetroLunNoRedundantIdList))
            return genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                                             constants.ULTRA_HYPER_METRO_VLUN_NOT_PASS_TIPS,
                                             errParas)
        else:
            errParas.append(','.join(noRedundantPathLunIdList))
            errParas.append(','.join(hyperMetroLunNoRedundantIdList))
            return genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                                             constants.ULTRA_VLUN_NOT_PASS_TIPS,
                                             errParas)
def getPropProgress(prog):
    '''
    @summary: 返回恰当的进度信息
    :param prog:
    :return:
    '''
    if COEXISTENCE:
        return prog/2
    return prog
def checkRedundantLinkByMultipathd(context):
    '''
    @summary: 评估自带多路径管理的存储LUN是否存在冗余路径。
    @param context: 上下文
    @return:评估结果.
    '''
    global MULTIPATHCTRL
    isQrySucc, portNumbers, cliRet = util.getTargetPortNum(context)
    util.addItemProgress(context, getPropProgress(constants.PROG15))
    if not isQrySucc:
        MULTIPATHCTRL = False
        errParas = [constants.QRY_DEV_NAME_CMD, "messageKey:" + constants.ERR_MSG_KEY_QRY_TGT_PORT_NUM_FAILED]
        util.log.error(context, 'Query target port number reported by storage failed.')
        errMsgKey = constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                                  cliRet, errMsgKey, errParas)
    if "not found" in cliRet:
        MULTIPATHCTRL = False
        errParas = [constants.QRY_DEV_NAME_CMD, "messageKey:" + "eval.host.multipath.reduntpath.isscsi.no.install"]
        util.log.error(context, 'Query target port number reported by storage failed.')
        errMsgKey = "eval.host.multipath.reduntpath.recommended.actions.isscsi.no.install"
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                                  cliRet, errMsgKey, errParas)

    if not portNumbers:
        errParas = [constants.QRY_DEV_NAME_CMD]
        MULTIPATHCTRL = False
        util.log.error(context, 'Query no array managed by ultrapath.')
        errMsgKey = constants.ERR_MSG_KEY_QRY_TGT_PORT_NUM_EMPTY
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                                  cliRet, errMsgKey, errParas)

    util.log.debug(context, 'Target port number list:' + unicode(portNumbers))
    tgtPortNumPthMap, qryPthCliRet = util.getPortNumPathDict(context, portNumbers)
    cliRet += ('\n' + qryPthCliRet)
    if not tgtPortNumPthMap:
        if COEXISTENCE:
            errParas = [",".join(constants.QRY_TGT_PORT_PATH_CMD % protNumber for protNumber in portNumbers),
                        "messageKey:" + constants.ERR_MSG_KEY_QRY_TGT_PORT_PATH_EMPTY]
        else:
            errParas = ["\",\n\"".join(constants.QRY_TGT_PORT_PATH_CMD % protNumber for protNumber in portNumbers),
                        "messageKey:" + constants.ERR_MSG_KEY_QRY_TGT_PORT_PATH_EMPTY]

        errMsgKey = constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION
        util.log.error(context, 'Query all the path of target port failed.')
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                                  cliRet, errMsgKey, errParas)

    util.log.debug(context, 'Target port number path:' + unicode(tgtPortNumPthMap))
    return evaluteAllDiskManagedByMultiPathd(context, tgtPortNumPthMap)

def evaluteAllDiskManagedByMultiPathd(context, tgtPortNumPthMap):
    '''
    @summary: 根据查询到的端口路径映射关系，判断自带多路径服务multipathd管理的所有华为磁盘是否存在冗余链路。
    @param context: Python执行上下文.
    @param tgtPortNumPthMap: TGT端口和路径的映射字典。
    @return: isEvalSuccess, notRedundantDiskList, cliRet
    '''
    cli = context.get("SSH")
    notRedundantPathDiskList = []
    allPthRet = ''
    qryAllDiskPthCmd = constants.QRY_DISK_PTH_MANAGED_BY_MULTIPATHD_CMD
    try:
        allPthRet = cli.execCmdWithNoCheckResult(qryAllDiskPthCmd,
                                       constants.HOST_CMD_TIMEOUT,
                                       constants.LINUX_CMD_DFT_END_LIST)

        diskPathDict = parseAllDiskPth(allPthRet, tgtPortNumPthMap)
        if not diskPathDict:#no disk found, unable to evaluate.
            errParas = [qryAllDiskPthCmd, "messageKey:" + constants.ERR_MSG_MULTIPATHD_NO_HW_DISK_FOUND]
            errMsgKey = constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION
            return genEvalResultForMultipath(EvalResultEnum.FAILED,
                             allPthRet, errMsgKey, errParas)
        COEXIST_RATE = 2
        progRate = 1 if COEXISTENCE else COEXIST_RATE
        prgStep, stepsUnit = util.calcPerStepDetail(constants.PROG35 * progRate, len(diskPathDict))
        idx = 0
        for diskName in diskPathDict:
            util.log.debug(context, 'LunName:' + unicode(diskName)
                           + 'LunPath:' + unicode(diskPathDict.get(diskName)))
            if not util.hasRedundantPathForCtrlIdPath(context, diskPathDict.get(diskName), isNodeId = True):
                notRedundantPathDiskList.append(diskName)
            util.itemProgressIncr(context, idx, prgStep, stepsUnit)
            idx += 1
        if not notRedundantPathDiskList:
            return genEvalResultForMultipath(EvalResultEnum.PASSED, allPthRet, '')
        else:
            errParas = [str(cli.getHost())]
            errParas.append(','.join(notRedundantPathDiskList))
            return genEvalResultForMultipath(EvalResultEnum.FAILED,
                             allPthRet,
                            constants.NMP_NOT_PASS_TIPS,
                            errParas)
    except:
        errParas = [qryAllDiskPthCmd, "messageKey:" + constants.ERR_MSG_KEY_QRY_SINGLE_LUN_PATH_FAILED]
        errMsgKey = constants.RECOMMENDED_ACTIONS_NETWORK_CONNECTION
        util.log.error(context, 'Query disk path exception:' + traceback.format_exc())
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                              allPthRet, errMsgKey, errParas)

def parseAllDiskPth(allPthRet, tgtPortNumPthMap):
    '''
    @summary: 解析multipaht - ll命令回文，生成磁盘名称到磁盘路径的字典。
    @param allPthRet: multipaht - ll回文。
    @param tgtPortNumPthMap: TGT端口到路径的映射字典，如：{'0:0:0':'0A', '0:0:1':'0b'}。
    @return: 磁盘名称到磁盘路径的字典。如：{'dm-2': ['0c', '0d'], 'dm-11': ['0c', '0b']}
    '''
    tgtPortNumPattern = re.compile(constants.TARGET_PORT_NUM_PATTERN, re.I)
    newDiskPattern = re.compile(constants.NEW_HW_DISK_PATTERN, re.I)
    aboveStaPattern = re.compile('.+prio.+(active|enabled)', re.I)
    belowStaPattern = re.compile('ready', re.I)
    diskPathDict = {}
    lines = allPthRet.splitlines()
    maxIdx = len(lines)
    i = 0
    while i < maxIdx:
        diskNameMatchObj = newDiskPattern.search(lines[i])
        if diskNameMatchObj:
            diskName = diskNameMatchObj.group().split()[0].strip()
            j = i + 2
            while not newDiskPattern.search(lines[j]) and j + 1 < maxIdx:
                aboveStaMatchObj = aboveStaPattern.search(lines[j])
                if aboveStaMatchObj:#找到第一个状态值后，遍历查找其下的所有正常路径。
                    pthIdx = j + 1
                    tgtPortNumMatchObj = tgtPortNumPattern.search(lines[pthIdx])
                    while tgtPortNumMatchObj:#匹配到TGT端口号.
                        pathStaMatchObj = belowStaPattern.search(lines[pthIdx])
                        if pathStaMatchObj:#状态正常
                            tgtPortNum = tgtPortNumMatchObj.group()
                            path = tgtPortNumPthMap.get(tgtPortNum)
                            if diskName in diskPathDict:
                                diskPathDict.get(diskName).append(path)
                            else:
                                diskPathDict[diskName] = [path]
                        #Find next path in next line.
                        pthIdx += 1
                        tgtPortNumMatchObj = tgtPortNumPattern.search(lines[pthIdx])
                    else:#非路径行，下一行可能是状态行或者新的磁盘信息。
                        j = pthIdx
                else:
                    j += 1
            else:#Inner while loop end, to find a new disk.
                i = j
        else:
            i += 1
    return diskPathDict

def genEvalResultForMultipath(evalResult, cliRet, errMsgKey, paraList = None):
    '''
    @summary: Create an evaluate result object of Java ItemEvalResult type.
    @param evalResut: Evaluate result.
    @param errMsgKey: error message key
    @param cliRet: CLI return message.
    @param paraList:Error message parameter list of string type.
    @return: item evaluate result.
    '''
    itemEvalResult = util.genEvalItemObj(Item.MULTIPATH_CHECK_ID,
                                         evalResult,
                                         cliRet, errMsgKey, paraList)
    return itemEvalResult


CLI = None
LANGUAGE = None
CHECK_FAIL = 0
CHECK_PASS = 1
cmd_info_id = ["cmd_info_multipath_device_mapper"  ,
               "cmd_info_multipath_multipath"      ,
               "cmd_info_multipath_version_huawei",
               "cmd_info_multipath_path_huawei",
               "cmd_info_multipath_showvlun_typeAll"
               ]

cmd_info_desc = ["rpm -qa |grep device-mapper",
                 "multipath -ll",
                 "upadmin show version",
                 "upadmin show vlun",
                 "upadmin show vlun type=all"
                 ]



# 取中间一个有效值
def check_and_get_result((fun_flag, result, fun_err_msg)):
    """
    :rtype : object
    """
    return result

# 取一个有效值
def check_storage_and_get_result((fun_flag, result, fun_err_msg)):
    """
    :rtype : object
    """
    return result

def cmd_execute(cmd_display, CLI, LANGUAGE):
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    for rg in range(len(cmd_info_id)):
        cmd_display_temp = CLI.execCmdHasLogTimout(cmd_info_desc[rg], 60)
        cmd_display.put("cmd_display" + (cmd_info_id[rg])[8:], cmd_display_temp)
        if None == cmd_display_temp or '' == cmd_display_temp or cmd_display_temp.find('TOOLKIT_SEND_CMD_TIME_OUT') > 0 or cmd_display_temp.find('TOOLKIT_EXE_CMD_FAILED') > 0:
            if "en" == LANGUAGE:
                fun_err_msg += cmd_info_desc[rg] + ":\texecute failed\r\n"
            else:
                fun_err_msg += cmd_info_desc[rg] + u":\t执行失败\r\n"
        else:
            if "en" == LANGUAGE:
                fun_err_msg += cmd_info_desc[rg] + ":\texecute success\r\n"
            else:
                fun_err_msg += cmd_info_desc[rg] + u":\t执行成功\r\n"
    cmd_display.put("err_msg", fun_err_msg)
    fun_flag = CHECK_PASS
    return fun_flag, cmd_display, fun_err_msg
