# coding=utf-8
__author__ = '******'
# 数据迁移: 获取linux主机:集群基本信息
import re
CLI = None
LANGUAGE = None
import os
import sys
CURRENT_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(CURRENT_PATH)
CURRENT_PATH = os.path.join(CURRENT_PATH, "..\\..")
sys.path.append(CURRENT_PATH)
HOST_CMD_TIMEOUT = 5 * 60
from common import util
from common import contentParse
from common import constants
ALUA_ENABLED='enabled'
ALUA_NOT_ENABLED = 'not enabled'
ALUA_FIGURE_UNKNOWN = 'unknown'
FORMAT_CMD_MAP = {'id':'solaris_alua_format' ,
                  'cmd' : r'format'}
MPIOCHECK_CMD_MAP_OLD = {'id':'solaris_alua_mpathadm_show_lun' ,
                         'cmd' : r' mpathadm show lu /dev/dsk/'}
def execute(context):
    '''
    Function describe  : collect Solaris host Alua information
    Return             : cmd display
    '''
    diskAluaChecker = DiskAluaChecker(context)
    return diskAluaChecker.doGrab()


class DiskAluaChecker(object):
    '''
    @summary: collect disk alua feature configuration on solaris server host
    '''

    def __init__(self, context):
        '''
        @summary: constructor
        '''
        self.context = context
        self.sshCon = self.context.get("SSH")
        self.lang = self.context.get("lang")
        self.cmd_display = self.context.get("ret_map")

    def doGrab(self):
        '''
        @summary: evaluation entrance
        '''
        resultStatus = ""
        failedItems = []
        util.log.info(self.context, "[Solaris] start to fetch disk format configuration.")
        cmdEchos = ""
        cmdArgs = {}
        cmdArgs['timeout'] = constants.LINUX_UPSCAN_CMD_TIMEOUT
        cmdArgs['recordLog'] = True
        cmdArgs['_SPEFIC_END_STRS'] = ['(enter its number):']
        cmdArgs['needCheckResult'] = False
        cmdEcho = self.sshCon.execCmdWithConditions(FORMAT_CMD_MAP['cmd'], cmdArgs)
        cmdEchos += cmdEcho
        self.cmd_display.put("cmd_display_" + FORMAT_CMD_MAP['id'], cmdEcho)
        status, result = self.getResultMsg(FORMAT_CMD_MAP['cmd'], cmdEcho)
        resultStatus += result
        if not status:
            self.cmd_display.put("err_msg", resultStatus)
            #record that current alua status collection has failed
            contentParse .saveAluaStatusAndCliRet2ForeEval (ALUA_FIGURE_UNKNOWN, cmdEchos, self.context)
            return self.cmd_display
        #send Ctrl+C if timeout
        self.sshCon.execCmdWithTimout("0", 1)
        diskNoList = self.parse4HuaweiDiskWwn(cmdEcho)
        
        if len(diskNoList) < 1:
            util.log.info(self.context, "[Solaris] current host has no huawei disk detected..")
            self.cmd_display.put("err_msg", resultStatus)
            #set current Alua status as false and execution succeeded
            contentParse .saveAluaStatusAndCliRet2ForeEval (ALUA_NOT_ENABLED, cmdEchos, self.context)
            return self.cmd_display

        util.log.info(self.context, "[Solaris] found huawei disks listed:" + ','.join(diskNoList))
        for diskNo in diskNoList:
            cmd = MPIOCHECK_CMD_MAP_OLD["cmd"] + diskNo + "s2"
            util.log.info(self.context, "[Solaris] cmd:" + cmd)
            cmdEcho = self.sshCon.execCmdWithTimout(cmd, constants.HOST_CMD_TIMEOUT)
            cmdEchos += cmdEcho
            self.cmd_display.put("cmd_display_" + MPIOCHECK_CMD_MAP_OLD['id'] + diskNo, cmdEcho)
            status, result = self.getResultMsg(cmd, cmdEcho)
            resultStatus += result
            if not status:
                failedItems.append(diskNo)
                continue
            for line in cmdEcho.splitlines():
                if "asymmetric" in line.lower():
                    util.log.info(self.context, "[Solaris] found current ALUA configuration: %s." % line)
                    items = line.split(":")
                    if len(items) > 1 and items[1].lower().strip() == 'yes':
                        util.log.info(self.context, "[Solaris] found lun with ALUA working: %s." % line)
                        contentParse .saveAluaStatusAndCliRet2ForeEval (ALUA_ENABLED, cmdEchos, self.context)
                        self.cmd_display.put("err_msg", resultStatus)
                        return self.cmd_display
                    else:
                        break
        if failedItems:
            util.log.info(self.context, "[solaris]found disk failed querying ALUA info")
            contentParse .saveAluaStatusAndCliRet2ForeEval (ALUA_FIGURE_UNKNOWN, cmdEchos, self.context)
        else:
            util.log.info(self.context, "[solaris]found all specified disk  ALUA not opened")
            contentParse .saveAluaStatusAndCliRet2ForeEval (ALUA_NOT_ENABLED, cmdEchos, self.context)

        self.cmd_display.put("err_msg", resultStatus)
        return self.cmd_display

    def parse4HuaweiDiskWwn(self, cmdEcho):
        '''
        @summary: parse 4 huawei's disk logic number
        @return: diskNo. list
        '''
        huaweiDisks = []
        isParseStart = False
        for  line in cmdEcho.splitlines():
            if "available disk selections:" in line.lower():
                isParseStart = True
                continue
            if isParseStart and "<" in line and ">" in line:
                if "huawei" in line.lower():
                    lineParams = line.split()
                    if len(lineParams) > 2:
                        huaweiDisks.append(lineParams[1])
            if "(enter its number):" in line:
                break
        return huaweiDisks

    def getResultMsg(self, step_name, cmd_display_temp):
        '''
        @summary: get result status's message
        '''
        result_msg = ""
        result = True
        if None == cmd_display_temp or '' == cmd_display_temp or\
            'TOOLKIT_SEND_CMD_TIME_OUT' in  cmd_display_temp or\
                'TOOLKIT_EXE_CMD_FAILED' in cmd_display_temp:
            result = False

        if "en" == self.lang:
            if result:
                result_msg += step_name + ":\texecute success\r\n"
            else:
                result_msg += step_name + ":\texecute failed\r\n"
        else:
            if result:
                result_msg += step_name + u":\t执行成功\r\n"
            else:
                result_msg += step_name + u":\t执行失败\r\n"

        return result, result_msg
